/* Estimate w/h process:
     x(t) = z(t) + eps(t)
     z(t) = rho*z(t-1) + xsi(t)
   z = wage/hour
   x = observed 'z'
   eps = measurement error
   Estimate rho and var(xsi). Use GMM.
*/

New;
Library maxlik , /*nlsys,*/ optmum;
/*#include gradhess.prg;*/


  /* Load data calculated from PSID 1988-1992
     Col1 = 1988 etc. 1592 individuals observed. */
load wh[1592,5] = c:\jesper\gauss\wh.asc;

/* Generate data */
/*
rho = 0.9;
sigme = sqrt(0.04);
sigmx = sqrt(0.05);
NN = 1592;
wh = zeros(NN,5);
xx = zeros(NN,1);
i = 2;
do while i <= 1005;
  xx = rho * xx + sigme * rndn(NN,1);
  if i > 1000;
    wh[.,i-1000] = xx + sigmx * rndn(NN,1);
  endif;
  i = i + 1;
endo;
  */

/* Use diff. data */
wh = wh[.,2:5] - wh[.,1:4];


n = rows(wh);
mn = meanc(wh);
wh = wh - meanc(mn);

nMC = 7;            /* # of moment conditions used */

  /* Initialize parameters */
theta = {0.8, 0.03, 0.1};
W = eye(nMC);
/*load W;
load theta;*/

nIters = 7;
iter = 1;
Do While (iter <= nIters);

  /* Solve GMM */
  __title = "Minimize GMM Loss Function";
  _opgtol = 1E-5;
  _opparnm = "rho " | "s2x " | "s2e ";
  _opalgr = 2;
  /*_opstep = 1;*/
  _opgdmd = 1;
  _opgrdh = 1e-6;
  _grdh = 1e-6;
  _opgdprc = &_opgrad;
  {xx, ff, gg, rr} = optmum(&loss,theta);
  theta = xx;

  /* Update weighting matrix W */
  S = zeros(nMC,nMC);
  mc = momcond(wh,theta);
  i = 1;
  Do While i <= n;
    S = S + mc[i,.]' * mc[i,.] / n;
    i = i + 1;
  Endo;
  W = inv(S);

  iter = iter + 1;
Endo;

  /* Calculate std. errors of estimates */
  /* See Hamilton eqs. 14.2.5-14.2.7 */
D = gradcd(&h,theta);
V = inv(D' * W * D);
V = V / n;
se = sqrt(diag(V));

print "theta   std.err.";
print (theta ~ se);


/******* END OF PROGRAM *******/


/* Calculate loss */
Proc loss(theta);
  local h;
  h = meanc(momcond(wh,theta));
  retp(h' * W * h);
Endp;

/* Calculate moments */
Proc h(theta);
  retp(meanc(momcond(wh,theta)));
Endp;

/* Calculate moment conditions used */
Proc momcond(wh,theta);

  local n,mom,mom_cond,rho,sigma2x,sigma2e,VZ;

  n = rows(wh);
  rho = theta[1];
  sigma2x = theta[2];
  sigma2e = theta[3];

  mom = zeros(n,nMC);

  /* Variances */
  mom[.,1:4] = wh.^2;

  /* Covariances */
  mom[.,5] = wh[.,1] .* wh[.,2];

  /* Covariances  */
  mom[.,6] = wh[.,2] .* wh[.,3];

  /* Covariances */
  mom[.,7] = wh[.,3] .* wh[.,4];

  mom_cond = zeros(n,nMC);

  VZ = 2 * sigma2x / (1+rho);

  /* First 4 unconditional variance moments. */
  mom_cond[.,1] = mom[.,1] - VZ - 2*sigma2e;
  mom_cond[.,2] = mom[.,2] - VZ - 2*sigma2e;
  mom_cond[.,3] = mom[.,3] - VZ - 2*sigma2e;
  mom_cond[.,4] = mom[.,4] - VZ - 2*sigma2e;

  /* Then 4 covariance moments */
  mom_cond[.,5] = mom[.,5] + sigma2x * (1-rho)/(1+rho) + sigma2e;
  mom_cond[.,6] = mom[.,6] + sigma2x * (1-rho)/(1+rho) + sigma2e;
  mom_cond[.,7] = mom[.,7] + sigma2x * (1-rho)/(1+rho) + sigma2e;

  retp(mom_cond);

Endp;


Proc _opgrad(x0);
    local f;

    f = &loss;
    retp(gradp(f,x0));
Endp;



